@@ -4,7 +4,7 @@ use warnings;
 use 5.008003;
 use Pegex::Base -base;
 
-our $VERSION = '0.05';
+our $VERSION = '0.10';
 our @EXPORT = qw(pegex);
 
 has 'grammar';
@@ -20,6 +20,12 @@ sub pegex {
     );
 }
 
+sub compile {
+    my $self = shift;
+    $self->grammar->compile;
+    return $self;
+}
+
 sub parse {
     my $self = shift;
     die 'Pegex->parse() takes one or two arguments ($input, $start_rule)'
