@@ -2,126 +2,107 @@ package Pegex::Compiler;
 use Pegex::Base -base;
 
 has 'grammar';
-has 'combined';
+has 'grammar_combined';
+has 'debug' => 0;
 
-sub grammar_file_to_yaml {
-    require YAML::XS;
-    my $class = shift;
-    my $file = shift;
-    open IN, $file or die "Can't open '$file'";
-    my $grammar = do {local $/; <IN>};
-    return YAML::XS::Dump($class->new->compile($grammar)->grammar);
-}
+has 'stack' => [];
 
-my $atoms = {
-    ALWAYS  => '',
-    NEVER   => '(?!)',
-    ALL     => '[\s\S]',
-    ANY     => '.',
-    BLANK   => '[\ \t]',
-    BLANKS  => '\ \t',
-    SPACE   => ' ',
-    TAB     => '\t',
-    WS      => '\s',
-    BREAK   => '\n',
-    CR      => '\r',
-    EOL     => '\r?\n',
-    ALPHA   => '[a-zA-Z]',
-    LOWER   => '[a-z]',
-    UPPER   => '[A-Z]',
-    DIGIT   => '[0-9]',
-    XDIGIT  => '[0-9a-fA-F]',
-    ALNUM   => '[a-zA-Z0-9]',
-    WORD    => '\w',
-
-    SINGLE  => "'",
-    DOUBLE  => '"',
-    LPAREN  => '\(',
-    RPAREN  => '\)',
-    LSQUARE => '\[',
-    RSQUARE => '\]',
-    LANGLE  => '<',
-    RANGLE  => '>',
-
-    BANG    => '!',
-    AT      => '\@',
-    HASH    => '\#',
-    DOLLAR  => '\$',
-    PERCENT => '%',
-    CARET   => '\^',
-    AMP     => '&',
-    STAR    => '\*',
-
-    TILDE   => '~',
-    GRAVE   => '`',
-    UNDER   => '_',
-    DASH    => '-',
-    PLUS    => '\+',
-    EQUAL   => '=',
-    PIPE    => '\|',
-    BACK    => '\\\\',
-    COLON   => ':',
-    SEMI    => ';',
-    COMMA   => ',',
-    DOT     => '\.',
-    QMARK   => '\?',
-    SLASH   => '\/',
-};
+my $atoms;
 
 sub compile {
     my $self = shift;
     $self = $self->new unless ref $self;
     my $grammar_text = shift;
     $self->grammar({});
-    $grammar_text =~ s/^#.*\n+//gm;
-    my $first_rule;
-    for my $rule (split /(?=^\w+:\s)/m, $grammar_text) {
-        (my $value = $rule) =~ s/^(\w+):// or die;
-        my $key = $1;
-        $value =~ s/\s+/ /g;
-        $value =~ s/^\s*(.*?)\s*$/$1/;
-        $self->grammar->{$key} = $value;
-        $first_rule ||= $key;
-    }
 
-    for my $rule (sort keys %{$self->grammar}) {
-        my $text = $self->grammar->{$rule};
-        my @tokens = ($text =~ m{(
-            /[^/]*/ |
-            <[\!\&]?\w+>[\?\*\+]? |
-            `[^`]*` |
-            \| |
-            \[[\!\&?]? |
-            \][\?\*\+]? |
-            \([\!\&?]? |
-            \)[\?\*\+]?
-        )}gx);
-        die "No tokens found for rule <$rule> => '$text'"
-            unless @tokens;
-        unshift @tokens, '[';
-        push @tokens, ']';
-        my $tree = $self->make_tree(\@tokens);
-        $self->grammar->{$rule} = $self->compile_next($tree);  
-    }
-    $self->combinate($first_rule);
-    $self->grammar->{_FIRST_RULE} = $first_rule;
+    require Pegex::Compiler::Grammar;
+    my $grammar = Pegex::Compiler::Grammar->new(
+        receiver => $self,
+        debug => $self->debug,
+    );
+#     $self->{parser} = $grammar;
+
+    $grammar->parse($grammar_text);
+
     return $self;
 }
 
+sub got_rule_name {
+    my $self = shift;
+    my $name = shift;
+    $self->grammar->{_FIRST_RULE} ||= $name;
+    push @{$self->stack}, [$name];
+}
+
+sub got_rule_definition {
+    my $self = shift;
+    $self->grammar->{$self->stack->[0]->[0]} = $self->stack->[0]->[1];
+    $self->stack([]);
+}
+
+sub got_regular_expression {
+    my $self = shift;
+    my $re = shift;
+    push @{$self->stack->[-1]}, {'+re' => $re};
+}
+
+sub try_any_group {
+    my $self = shift;
+    push @{$self->stack}, {'+any' => []};
+}
+sub not_any_group {
+    my $self = shift;
+    pop @{$self->stack};
+}
+
+sub try_all_group {
+    my $self = shift;
+    push @{$self->stack}, {'+all' => []};
+}
+sub not_all_group {
+    my $self = shift;
+    pop @{$self->stack};
+}
+
+sub got_rule_group {
+    my $self = shift;
+    my $group = pop @{$self->stack};
+    push @{$self->stack->[-1]}, $group;
+}
+
+sub got_rule_reference {
+    my $self = shift;
+    my ($modifier, $name, $quantifier) = @_;
+    my $rule = {
+        '+rule' => $name,
+    };
+    $rule->{'<'} = $quantifier if $quantifier;
+    my $current = $self->stack->[-1];
+    push @{$current->{'+all'}}, $rule
+        if $current->{'+all'};
+    push @{$current->{'+any'}}, $rule
+        if $current->{'+any'};
+}
+
+
+# Combination
 sub combinate {
     my $self = shift;
-    my $rule = shift;
-    $self->combined({});
+    my $rule = shift || $self->grammar->{_FIRST_RULE};
+    $self->grammar_combined({
+        map {($_, $self->grammar->{$_})} grep { /^_/ } keys %{$self->grammar}
+    });
     $self->combinate_rule($rule);
-    $self->grammar($self->combined);
+    $self->grammar($self->grammar_combined);
+    return $self;
 }
 
 sub combinate_rule {
     my $self = shift;
     my $rule = shift;
-    return if exists $self->combined->{$rule};
+    return if exists $self->grammar_combined->{$rule};
 
-    my $object = $self->combined->{$rule} = $self->grammar->{$rule};
+    my $object = $self->grammar_combined->{$rule} = $self->grammar->{$rule};
     $self->combinate_object($object);
 }
 
@@ -156,7 +137,8 @@ sub combinate_object {
     elsif (exists $object->{'+error' }) {
     }
     else {
-        die "Can't combinate: $object";
+        require YAML::XS;
+        die "Can't combinate:\n" . YAML::XS::Dump($object);
     }
 }
 
@@ -176,119 +158,39 @@ sub combinate_re {
     }
 }
 
-sub make_tree {
+sub compile_file {
     my $self = shift;
-    my $tokens = shift;
-    my $stack = [];
-    my $tree = [];
-    push @$stack, $tree;
-    for my $token (@$tokens) {
-        if ($token =~ /^[\[\(]/) {
-            push @$stack, [];
-        }
-        push @{$stack->[-1]}, $token;
-        if ($token =~ /^[\]\)]/) {
-            my $branch = pop @$stack;
-            push @{$stack->[-1]}, $branch;
-        }
-    }
-    return $tree->[0];
+    my $file = shift;
+    open IN, $file or die "Can't open '$file'";
+    my $grammar = do {local $/; <IN>};
+    $self->compile($grammar);
+    return $self;
 }
 
-sub compile_next {
+sub to_yaml {
+    require YAML::XS;
     my $self = shift;
-    my $node = shift;
-    my $unit = ref($node)
-        ? $node->[2] eq '|'
-            ? $self->compile_group($node, 'any')
-            : $self->compile_group($node, 'all')
-        : $node =~ m!/! ? $self->compile_re($node)
-        : $node =~ m!<! ? $self->compile_rule($node)
-        : $node =~ m!`! ? $self->compile_error($node)
-        : die $node;
-
-    while (defined $unit->{'+all'} and @{$unit->{'+all'}} == 1) {
-        $unit = $unit->{'+all'}->[0];
-    }
-    return $unit;
+    return YAML::XS::Dump($self->grammar);
 }
 
-sub compile_group {
+sub to_json {
+    require JSON::XS;
     my $self = shift;
-    my $node = shift;
-    my $type = shift;
-    die unless @$node > 2;
-    if ($node->[0] =~ s/\&$//) {
-        return $self->compile_has($node);
-    }
-    if ($node->[0] =~ s/\!$//) {
-        return $self->compile_not($node);
-    }
-    my $object = {};
-    if ($node->[-1] =~ /([\?\*\+])$/) {
-        $object->{'<'} = $1;
-    }
-    shift @$node;
-    pop @$node;
-    if ($type eq 'any') {
-        $object->{'+any'} = [
-            map $self->compile_next($_), grep {$_ ne '|'} @$node
-        ];
-    }
-    elsif ($type eq 'all') {
-        $object->{'+all'} = [
-            map $self->compile_next($_), @$node
-        ];
-    }
-    return $object;
-}
-
-sub compile_re {
-    my $self = shift;
-    my $node = shift;
-    my $object = {};
-    $node =~ s!^/(.*)/$!$1! or die $node;
-    $object->{'+re'} = $node;
-    return $object;
-}
-
-sub compile_rule {
-    my $self = shift;
-    my $node = shift;
-    my $object = {};
-    if ($node =~ s/([\?\*\+])$//) {
-        $object->{'<'} = $1;
-    }
-    $node =~ s!^<(.*)>$!$1! or die;
-    if ($node =~ s/^!//) {
-        $object->{'+not'} = $node;
-    }
-    else {
-        $object->{'+rule'} = $node;
-    }
-    return $object;
-}
-
-sub compile_error {
-    my $self = shift;
-    my $node = shift;
-    my $object = {};
-    $node =~ s!^`(.*)`$!$1! or die $node;
-    $object->{'+error'} = $node;
-    return $object;
+    return JSON::XS->new->utf8->canonical->pretty->encode($self->grammar);
 }
 
 sub to_perl {
     my $self = shift;
-    $self->compile_perl_regex($self->grammar);
+    $self->perl_regexes($self->grammar);
     require Data::Dumper;
+    no warnings 'once';
     $Data::Dumper::Terse = 1;
     $Data::Dumper::Indent = 1;
     $Data::Dumper::Sortkeys = 1;
     return Data::Dumper::Dumper($self->grammar);
 }
 
-sub compile_perl_regex {
+sub perl_regexes {
     my $self = shift;
     my $node = shift;
     if (ref($node) eq 'HASH') {
@@ -298,13 +200,71 @@ sub compile_perl_regex {
         }
         else {
             for (keys %$node) {
-                $self->compile_perl_regex($node->{$_});
+                $self->perl_regexes($node->{$_});
             }
         }
     }
     elsif (ref($node) eq 'ARRAY') {
-        $self->compile_perl_regex($_) for @$node;
+        $self->perl_regexes($_) for @$node;
     }
 }
 
-1;
+$atoms = {
+    ALWAYS  => '',
+    NEVER   => '(?!)',
+    ALL     => '[\s\S]',
+    ANY     => '.',
+    BLANK   => '[\ \t]',
+    BLANKS  => '\ \t',
+    SPACE   => ' ',
+    TAB     => '\t',
+    WS      => '\s',
+    BREAK   => '\n',
+    CR      => '\r',
+    EOL     => '\r?\n',
+    EOS     => '\z',
+    ALPHA   => '[a-zA-Z]',
+    LOWER   => '[a-z]',
+    UPPER   => '[A-Z]',
+    DIGIT   => '[0-9]',
+    XDIGIT  => '[0-9a-fA-F]',
+    ALNUM   => '[a-zA-Z0-9]',
+    WORD    => '\w',
+
+    SINGLE  => "'",
+    DOUBLE  => '"',
+    LPAREN  => '\(',
+    RPAREN  => '\)',
+    LCURLY  => '\{',
+    RCURLY  => '\}',
+    LSQUARE => '\[',
+    RSQUARE => '\]',
+    LANGLE  => '<',
+    RANGLE  => '>',
+
+    BANG    => '!',
+    AT      => '\@',
+    HASH    => '\#',
+    DOLLAR  => '\$',
+    PERCENT => '%',
+    CARET   => '\^',
+    AMP     => '&',
+    STAR    => '\*',
+
+    TILDE   => '~',
+    GRAVE   => '`',
+    UNDER   => '_',
+    DASH    => '-',
+    PLUS    => '\+',
+    EQUAL   => '=',
+    PIPE    => '\|',
+    BACK    => '\\\\',
+    COLON   => ':',
+    SEMI    => ';',
+    COMMA   => ',',
+    DOT     => '\.',
+    QMARK   => '\?',
+    SLASH   => '/',
+};
+
+sub atoms { return $atoms }
